/*
	FixIMEI:
		This program try to fix the IMEI of a MPX200.
		It was confirmed that the IMEI check was related to the last 256KB rom of MPX200.
		And only 8 bytes data need to fix to get the IMEI back.
		The program first read a template 256KB rom file(haha.raw), this 256KB rom has unlock data and something else.
		Then locate the specical record related to IMEI check.
		And then it read the user 256KB rom file(input by user/dump from a user MPX200), and locate the special record
		related to IMEI check.
		The program replace the template rom's IMEI related check record(38 bytes record) with the user rom's.
		It is confirmed that the final written out 256KB rom file contain the user special IMEI info and unlock data.
		
		The file generated by this program should be used with other tools to update to the MPX200.
		
	Usage:
		FixIMEI user_special_input_rom_file output_rom_file
		
		user_special_input_rom_file: 256KB rom file dump by MPX200_Backup program, in BIN/IMG/RAW format.
		output_rom_file: output 256KB rom file, which contain the IMEI info from user_special_input_rom_file and unlock data.
		
	By marsgod 2005-04-08
*/

#include <stdio.h>
#include <stdlib.h>

long search_binstring(unsigned char *);

int main(int argc, char *argv[])
{
	FILE *ftemp;
	
	unsigned char *temp, *buff;
	
	long len;
	long pos1,pos2;
	int i;
	
	if (argc!=3)
	{
		printf("Usage: %s user_special_input_rom_file output_rom_file\n",argv[0]);
		return 1;
	}
	
	// read the template file
	ftemp = fopen("haha.raw","rb");
	
	if (ftemp==NULL)
	{
		printf("Error reading the template file :haha.raw\n");
		return 1;
	}
	
	temp = (unsigned char *)malloc(257*1024);
	if (temp==NULL)
	{
		printf("Error alloc memory 257KB\n");
		fclose(ftemp);
		return 1;
	}
	
	len = fread(temp,1,256*1024,ftemp);
	if (len!=256*1024)
	{
		printf("haha.raw corrupt? should be 256KB %d\n",len);
		free(temp);
		fclose(ftemp);
		return 1;
	}
	
	fclose(ftemp);
	
	// search template memory for string 55AA3E858700FFFFFFFF
	pos1 = search_binstring(temp);
	if (pos1 == -1)
	{
		printf("haha.raw corrupt? can't find the bin string\n");
		free(temp);
		return 1;
	}
	
	// read the orig file
	ftemp = fopen(argv[1],"rb");
	
	if (ftemp==NULL)
	{
		printf("Error reading the input file :%s\n",argv[1]);
		free(temp);
		return 1;
	}
	
	buff = (unsigned char *)malloc(257*1024);
	if (buff==NULL)
	{
		printf("Error alloc memory 257KB\n");
		fclose(ftemp);
		free(temp);
		return 1;
	}
	
	len = fread(buff,1,256*1024,ftemp);
	if (len!=256*1024)
	{
		printf("%s corrupt? should be 256KB\n",argv[1]);
		free(temp);
		free(buff);
		fclose(ftemp);
		return 1;
	}
	fclose(ftemp);
	
	// search orig memory for string 55AA3E858700FFFFFFFF
	pos2 = search_binstring(buff);
	if (pos2 == -1)
	{
		printf("%s corrupt? can't find the bin string\n",argv[1]);
		free(buff);
		free(temp);
		return 1;
	}
	
	// replace the template memory with the special record from the orig file
	for (i=0;i<38;i++)
	{
		temp[pos1+i] = buff[pos2+i];
	}
	
	// write out the modified template file, unlock, with IMEI
	ftemp = fopen(argv[2],"wb");
	
	if (ftemp==NULL)
	{
		printf("Error writing the output file :%s\n",argv[2]);
		free(buff);
		free(temp);
		return 1;
	}
	
	len = fwrite(temp,1,256*1024,ftemp);
	if (len!=256*1024)
	{
		printf("%s corrupt? should be 256KB\n",argv[2]);
		free(temp);
		free(buff);
		fclose(ftemp);
		return 1;
	}
	
	fclose(ftemp);
	free(temp);
	free(buff);
	return 0;
}

long search_binstring(unsigned char *buff)
{
	// search memory for string 55AA3E858700FFFFFFFF
	long pos;
	long temp;
	int flag = 0;
	long i;
	
	pos = -1;
	
	temp = 0;
	
	for (i=0;i<256*1024;i++)
	{
		if (buff[i] == 0x55 && flag == 0)
		{
			temp = i;
			flag = 1;
		}
		else
		if (buff[i] == 0xAA && flag == 1)
			flag = 2;
		else
		if (buff[i] == 0x3E && flag == 2)
			flag = 3;
		else
		if (buff[i] == 0x85 && flag == 3)
			flag = 4;
		else
		if (buff[i] == 0x87 && flag == 4)
			flag = 5;
		else
		if (buff[i] == 0x00 && flag == 5)
			flag = 6;
		else
		if (buff[i] == 0xFF && flag == 6)
			flag = 7;
		else
		if (buff[i] == 0xFF && flag == 7)
			flag = 8;
		else
		if (buff[i] == 0xFF && flag == 8)
			flag = 9;
		else
		if (buff[i] == 0xFF && flag == 9)
		{
			pos = temp;
			break;
		}
		else
			flag = 0;
	}
	
	return pos;
}
