/*
	raw2bin:
		This program is used to convert a 256KB rom raw file into a bin format file only.
		Please note, the program only convert the raw file and locate it to the address 0x83BC0000!!
		Also, the converted bin file will be filled the end of file with 0xFF to the 4096 boundary, this is a special need 
		by the MPX200.
		
		The file generated by this program should be used with other tools to update to the MPX200.
		
	Usage:
		raw2bin input_raw_file output_bin_file
		input_raw_file: the 256KB rom file dump by MPX200_Backup or generated by FixIMEI, in raw format
		output_bin_file: output 256KB rom file, in bin format.
	
	By marsgod 2005-04-08
*/

#include <stdio.h>
#include <stdlib.h>

unsigned char memory[32768L*1024L];

int main(int argc,char *argv[])
{
	FILE *fp;
	FILE *rp1;
	unsigned int i,j;
	unsigned int len1,len2,len3;
	unsigned int checksum;
	
	unsigned int tmp;
	unsigned int block_num,block_base;
	
	if (argc != 3)
	{
		printf("Usage: %s input_raw_file output_bin_file\n",argv[0]);
		return 1;
	}
	
	fp = fopen(argv[2],"wb");
	if (fp==NULL)
	{
		printf("Can't open bin file %s to write!\n",argv[2]);
		return 1;
	}
	
	
	rp1 = fopen(argv[1],"rb");
	if (rp1==NULL)
	{
		printf("Can't open raw file %s to read!\n",argv[1]);
		fclose(fp);
		return 1;
	}
	
	fseek(rp1,0,SEEK_END);

	len1 = ftell(rp1);
	
	printf("len1=%d\n",len1);
	
	if (len1%131072!=0)
		printf("Warning! Input raw file not align at 4KB address....\n");
	
	fseek(rp1,0,SEEK_SET);
	
	fprintf(fp,"B000FF\n");
	
	tmp = 0x83BC0000;
	fwrite(&tmp,1,sizeof(unsigned int),fp);
	tmp = len1;
	fwrite(&tmp,1,sizeof(unsigned int),fp);
	
	// write the blocks
	
	block_num = len1 / 131072;
	block_base = 0x83BC0000;
	
	for (i=0;i<block_num;i++)
	{
		tmp = block_base + i*131072;
		fwrite(&tmp,1,sizeof(unsigned int),fp);
		tmp = 131072;
		fwrite(&tmp,1,sizeof(unsigned int),fp);
		
		fread(memory,1,131072,rp1);
		
		checksum = 0;
		for (j=0;j<131072;j++)
		{
			checksum = checksum + memory[j];
		}
		
		fwrite(&checksum,1,sizeof(unsigned int),fp);
		fwrite(memory,1,131072,fp);
		
	}
	
	// write the last block
	
	tmp = 0x0;
	fwrite(&tmp,1,sizeof(unsigned int),fp);
	tmp = 0x0;
	fwrite(&tmp,1,sizeof(unsigned int),fp);
	tmp = 0x0;
	fwrite(&tmp,1,sizeof(unsigned int),fp);
	
	
	// fill the end of file with 0xFF to the 4096 boundary
	len1 = ftell(fp);
	
	len2 = len1 % 4096;
	
	if (len2 != 0)
	{
		len3 = 4096 - len2;
		
		for (i=0;i<len3+1;i++)
			memory[i] = 0xFF;
			
		fwrite(memory,1,len3,fp);
	}
	

	fclose(rp1);	
	fclose(fp);
	
	return 0;
}

